/*
**	MiscPanel.c
**
**	Editing panel for miscellaneous parts of the configuration
**
**	Copyright  1990-1993 by Olaf `Olsen' Barthel & MXM
**		All Rights Reserved
*/

#include "termGlobal.h"

enum	{	GAD_PRIORITY,GAD_BACKUP,GAD_SHOWFASTMACROS,GAD_RELEASE,
		GAD_OVERRIDE,GAD_AUTOUPLOAD,GAD_SETBIT,GAD_TRANSFER_ICONS,GAD_IDENTIFY,
		GAD_USE,GAD_CANCEL,GAD_DEFAULT };

STATIC STRPTR IdentifyType[4];

STATIC struct Gadget *
CreateAllGadgets(struct Configuration *Config,struct Gadget **GadgetArray,struct Gadget **GadgetList,BYTE *Default)
{
	struct Gadget		*Gadget;
	struct NewGadget	 NewGadget;
	UWORD			 Counter = 0;

	SZ_SizeSetup(Window -> WScreen,&UserFont,TRUE);

	LocalizeString(IdentifyType,MSG_MISCPANEL_IDENTIFY_IGNORE_TXT,MSG_MISCPANEL_IDENTIFY_SOURCE_TXT);

	memset(&NewGadget,0,sizeof(struct NewGadget));

	if(Gadget = CreateContext(GadgetList))
	{
		LONG Max;

		SZ_AddLeftOffset(SZ_LeftOffsetDelta(MSG_MISCPANEL_PROGRAM_PRIORITY_GAD,MSG_MISCPANEL_IDENTIFY_FILES_GAD));

		SZ_ResetMaxWidth();

		SZ_UpdateMaxWidth(CYCLE_KIND,NULL,0,IdentifyType);
		SZ_UpdateMaxWidth(CHECKBOX_KIND,NULL,0,NULL);

		Max = SZ_ResetMaxWidth();

		if(Max < 140)
			Max = 140;

		SZ_SetWidth(Max);

		NewGadget . ng_GadgetText	= LocaleString(MSG_MISCPANEL_PROGRAM_PRIORITY_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(SLIDER_KIND,Gadget,&NewGadget,
			SZ_Adjust,		TRUE,
			SZ_AutoWidth,		TRUE,

			GTSL_Min,		-128,
			GTSL_Max,		127,
			GTSL_Level,		Config -> MiscConfig -> Priority,
			GTSL_MaxLevelLen,	4,
			GTSL_LevelFormat,	"%4ld",
			PGA_Freedom,		LORIENT_HORIZ,
			GA_Immediate,		TRUE,
			GA_RelVerify,		TRUE,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_MISCPANEL_BACKUP_CONFIGURATION_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(CHECKBOX_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,
			SZ_AlignExtra,	TRUE,

			GTCB_Checked,	Config -> MiscConfig -> BackupConfig,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_MISCPANEL_SHOW_FAST_MACROS_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(CHECKBOX_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,
			SZ_AlignExtra,	TRUE,

			GTCB_Checked,	Config -> MiscConfig -> OpenFastMacroPanel,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_MISCPANEL_RELEASE_SERIAL_DEVICE_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(CHECKBOX_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,
			SZ_AlignExtra,	TRUE,

			GTCB_Checked,	Config -> MiscConfig -> ReleaseDevice,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_MISCPANEL_OVERRIDE_TRANSFER_PATH_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(CHECKBOX_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,
			SZ_AlignExtra,	TRUE,

			GTCB_Checked,	Config -> MiscConfig -> OverridePath,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_MISCPANEL_USE_AUTO_UPLOAD_PANEL_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(CHECKBOX_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,

			GTCB_Checked,	Config -> MiscConfig -> AutoUpload,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_MISCPANEL_SET_ARCHIVED_BIT_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(CHECKBOX_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,

			GTCB_Checked,	Config -> MiscConfig -> SetArchivedBit,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_MISCPANEL_TRANSFER_ICONS_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(CHECKBOX_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,

			GTCB_Checked,	Config -> MiscConfig -> TransferIcons,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_MISCPANEL_IDENTIFY_FILES_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(CYCLE_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,

			GTCY_Labels,	IdentifyType,
			GTCY_Active,	Config -> MiscConfig -> IdentifyFiles,
		TAG_DONE);

		SZ_UpdateMaxWidth(BUTTON_KIND,LocaleString(MSG_GLOBAL_USE_GAD),0,NULL);
		SZ_UpdateMaxWidth(BUTTON_KIND,LocaleString(MSG_GLOBAL_CANCEL_GAD),0,NULL);
		SZ_UpdateMaxWidth(BUTTON_KIND,LocaleString(MSG_GLOBAL_DEFAULT_GAD),0,NULL);

		SZ_SetWidth(SZ_ResetMaxWidth());

		NewGadget . ng_GadgetText	= LocaleString(MSG_GLOBAL_USE_GAD);
		NewGadget . ng_GadgetID		= Counter;
		NewGadget . ng_Flags		= 0;

		GadgetArray[Counter++] = Gadget = CreateGadget(BUTTON_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,
			SZ_AlignExtra,	TRUE,
			SZ_AlignLeft,	TRUE,
			SZ_AlignBottom,	TRUE,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_GLOBAL_CANCEL_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(BUTTON_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,
			SZ_AlignRight,	TRUE,
		TAG_DONE);

		if(Default)
		{
			NewGadget . ng_GadgetText	= LocaleString(MSG_GLOBAL_DEFAULT_GAD);
			NewGadget . ng_GadgetID		= Counter;

			GadgetArray[Counter] = Gadget = CreateGadget(BUTTON_KIND,Gadget,&NewGadget,
				SZ_Adjust,	TRUE,
				SZ_AutoWidth,	TRUE,
				SZ_AlignCentre,	TRUE,
			TAG_DONE);
		}
	}

	return(Gadget);
}

BYTE
MiscPanel(struct Configuration *Config,BYTE *Default)
{
	struct Window	*PanelWindow;
	struct Gadget	*GadgetList = NULL;
	struct Gadget	*GadgetArray[GAD_DEFAULT + 1];

	SaveConfig(Config,PrivateConfig);

	if(CreateAllGadgets(Config,GadgetArray,&GadgetList,Default))
	{
		if(PanelWindow = OpenWindowTags(NULL,
			WA_Left,	GetScreenLeft(Window) + (GetScreenWidth(Window) - SZ_GetWindowWidth()) / 2,
			WA_Top,		GetScreenTop(Window) + (GetScreenHeight(Window) - SZ_GetWindowHeight())	/ 2,
			WA_Width,	SZ_GetWindowWidth(),
			WA_Height,	SZ_GetWindowHeight(),

			WA_Activate,	TRUE,
			WA_DragBar,	TRUE,
			WA_DepthGadget,	TRUE,
			WA_RMBTrap,	TRUE,
			WA_DepthGadget,	TRUE,
			WA_CloseGadget,	TRUE,
			WA_CustomScreen,Window -> WScreen,

			WA_IDCMP,	IDCMP_GADGETDOWN | IDCMP_ACTIVEWINDOW | IDCMP_RAWKEY | IDCMP_VANILLAKEY | IDCMP_CLOSEWINDOW | BUTTONIDCMP | CHECKBOXIDCMP | SLIDERIDCMP | INTEGERIDCMP,

			WA_Title,	LocaleString(MSG_MISCPANEL_MISCELLANEOUS_PREFERENCES_TXT),
		TAG_DONE))
		{
			struct IntuiMessage	*Massage;
			ULONG			 IClass,Code;
			struct Gadget		*Gadget;
			BYTE			 Terminated = FALSE;

			GuideContext(CONTEXT_MISC);

			PushWindow(PanelWindow);

			AddGList(PanelWindow,GadgetList,(UWORD)-1,(UWORD)-1,NULL);
			RefreshGList(GadgetList,PanelWindow,NULL,(UWORD)-1);
			GT_RefreshWindow(PanelWindow,NULL);

			ActiveGadget = NULL;

			SetSignal(0,SIG_BREAK);

			while(!Terminated)
			{
				if(Wait(PORTMASK(PanelWindow -> UserPort) | SIG_BREAK) & SIG_BREAK)
				{
					SaveConfig(PrivateConfig,Config);

					break;
				}

				while(!Terminated && (Massage = (struct IntuiMessage *)GT_GetIMsg(PanelWindow -> UserPort)))
				{
					IClass	= Massage -> Class;
					Code	= Massage -> Code;
					Gadget	= (struct Gadget *)Massage -> IAddress;

					GT_ReplyIMsg(Massage);

					KeySelect(GadgetArray,GAD_CANCEL,Code,PanelWindow,&Gadget,&IClass,&Code);

					if(IClass == IDCMP_GADGETDOWN)
					{
						if((Gadget -> GadgetType & GTYP_GTYPEMASK) == GTYP_STRGADGET)
							ActiveGadget = Gadget;
					}

					if(IClass == IDCMP_ACTIVEWINDOW && ActiveGadget)
						ActivateGadget(ActiveGadget,PanelWindow,NULL);

					if(IClass == IDCMP_CLOSEWINDOW)
					{
						SaveConfig(PrivateConfig,Config);

						Terminated = TRUE;
					}

					if(IClass == IDCMP_MOUSEMOVE)
					{
						switch(Gadget -> GadgetID)
						{
							case GAD_PRIORITY:

								Config -> MiscConfig -> Priority = (BYTE)Code;
								break;
						}
					}

					if(IClass == IDCMP_GADGETUP)
					{
						switch(Gadget -> GadgetID)
						{
							case GAD_PRIORITY:

								Config -> MiscConfig -> Priority = (BYTE)Code;
								break;

							case GAD_IDENTIFY:

								Config -> MiscConfig -> IdentifyFiles = Code;
								break;

							case GAD_USE:

								Terminated = TRUE;

								Config -> MiscConfig -> BackupConfig		= SZ_GetChecked(GadgetArray[GAD_BACKUP]);
								Config -> MiscConfig -> OpenFastMacroPanel	= SZ_GetChecked(GadgetArray[GAD_SHOWFASTMACROS]);
								Config -> MiscConfig -> ReleaseDevice		= SZ_GetChecked(GadgetArray[GAD_RELEASE]);
								Config -> MiscConfig -> OverridePath		= SZ_GetChecked(GadgetArray[GAD_OVERRIDE]);

								Config -> MiscConfig -> AutoUpload		= SZ_GetChecked(GadgetArray[GAD_AUTOUPLOAD]);
								Config -> MiscConfig -> SetArchivedBit		= SZ_GetChecked(GadgetArray[GAD_SETBIT]);
								Config -> MiscConfig -> TransferIcons		= SZ_GetChecked(GadgetArray[GAD_TRANSFER_ICONS]);

								break;

							case GAD_CANCEL:

								Terminated = TRUE;

								SaveConfig(PrivateConfig,Config);

								break;

							case GAD_DEFAULT:

								*Default	= TRUE;

								Terminated	= TRUE;

								break;
						}
					}
				}
			}

			RemoveGList(PanelWindow,GadgetList,(UWORD)-1);

			PopWindow();

			CloseWindow(PanelWindow);
		}
	}

	FreeGadgets(GadgetList);

	return((BYTE)memcmp(PrivateConfig -> MiscConfig,Config -> MiscConfig,sizeof(struct MiscSettings)));
}
